/*! @file gpr_log.c
 *
 *  @brief Implementation of functions used for logging
 *
 *  @version 1.0.0
 *
 *  (C) Copyright 2018 GoPro Inc (http://gopro.com/).
 *
 *  Licensed under either:
 *  - Apache License, Version 2.0, http://www.apache.org/licenses/LICENSE-2.0
 *  - MIT license, http://opensource.org/licenses/MIT
 *  at your option.
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

#include "timer.h"
#include "stdc_includes.h"

TIMER LogTimer;

bool LogInit(void)
{
    InitTimer(&LogTimer);

    return true;
}

#ifndef LogPrint
int LogPrint(const char* format, ... )
{
    StopTimer(&LogTimer);
    
	printf("[%5d-ms] ", (unsigned int)TimeMSecs(&LogTimer));

	{
		va_list argptr;
		va_start(argptr, format);

		vfprintf(stdout, format, argptr);

		va_end(argptr);
	}

    printf( "%c", '\n' );
    
    StartTimer(&LogTimer);
    
    return 0;
}
#endif // LogPrint

bool LogUninit(void)
{
    return true;
}
